#ifndef __RAYTRACING_H__
#define __RAYTRACING_H__

#include "matrices.h"
#include "primitives.h"
#include "QGLViewer/simple_viewer.h"
#include <thread>

#include <QApplication>
#include <QLabel>

const Vec3 ROUGE   = {1,0,0}; //0
const Vec3 VERT    = {0,1,0}; //1
const Vec3 BLEU    = {0,0,1}; //2
const Vec3 JAUNE   = {1,1,0}; //3
const Vec3 CYAN    = {0,1,1}; //4
const Vec3 MAGENTA = {1,0,1}; //5
const Vec3 BLANC   = {1,1,1}; //6
const Vec3 GRIS    = {0.5,0.5,0.5}; //7
const Vec3 NOIR    = {0,0,0}; //8
const Vec3 ROUGE2   = {0.5,0,0}; //9
const Vec3 VERT2    = {0,0.5,0}; //10
const Vec3 ORANGE   = {1,0.5,0}; //11

class Node;
struct Inter
{
	float a_min; //distance de l'intersection.
	int inter_dir; //direction de l'intersection.
	Node* node; //noeud intercepté, if any.
	Vec3 O; //point d'impact du rayon.
	Vec3 D; //direction du rayon. (ps : ctrl+shift+R sur qtcreator pour changer le nom des variables).
};

class Node
{
public:
	static const float Epsilon ;
	Mat4 transfo; //matrice de transformation.
	Mat4 inv_transfo;
	Vec3 Col; //couleur.
	float spec; //indice de spécularité.
	float transp; //indice de transparence.
	Node(const Mat4& m, const Vec3& c, float sp, float tr);
	inline virtual ~Node() {}
	virtual void draw_gl() const =0 ; //la fonction permettant de dessiner la primitive dans la vue temps réel.
	virtual bool intersecte(const Vec3& P, const Vec3& V, Inter* I)=0; //test d'intersection de la primitive (modifier contenu de I).
	virtual Vec3 normal(const Inter& I, const Vec3& P)=0; //la normale à la primitive

	static Primitives prim;
};

class Cube: public Node
{
public:
	inline Cube(const Mat4& m, const Vec3& c, float sp, float tr): Node(m,c,sp,tr) {}
	void draw_gl() const;
	bool intersecte(const Vec3& P, const Vec3& V, Inter* I);
	Vec3 normal(const Inter& I, const Vec3& P);
};

class Sphere: public Node
{
public:
	inline Sphere(const Mat4& m, const Vec3& c, float sp, float tr): Node(m,c,sp,tr) {}
	void draw_gl() const;
	bool intersecte(const Vec3& P, const Vec3& V, Inter* I);
	Vec3 normal(const Inter& I, const Vec3& P);
};


class Cylinder: public Node
{
public:
	inline Cylinder(const Mat4& m, const Vec3& c, float sp, float tr): Node(m,c,sp,tr) {}
	void draw_gl() const;
	bool intersecte(const Vec3& P, const Vec3& V, Inter* I);
	Vec3 normal(const Inter& I, const Vec3& P);
};

class BVH
{
public:
	Mat4 transfo;
	Mat4 inv_transfo;
	std::vector<BVH*> children;
	std::vector<Node*> nodes;

	inline BVH(const Mat4& m):
		transfo(m),inv_transfo(glm::inverse(m))
	{}

	inline BVH* add_child(const Mat4& m) {BVH* b = new BVH(m); children.push_back(b); return b;}

	inline void add_node(Node* n) {nodes.push_back(n);}

	template<typename F>
	void intersecte(const Vec3& P, const Vec3& V, Inter* I, F func);
};

class RTracer
{
public:
	BVH* bvh;
	SimpleViewer* sv;
	std::vector<Node*> primitives;
	int depth;
	Vec3 posLum;
	static const int nbthr = 8;

	RTracer(SimpleViewer* s);

	void add_cube_bvh(BVH* b, const Mat4& m, const Vec3& color, float spec, float tr);
	void add_sphere_bvh(BVH* b, const Mat4& m, const Vec3& color, float spec, float tr);
	void add_cylinder_bvh(BVH* b, const Mat4& m, const Vec3& color, float spec, float tr);

	QLabel* CalcImage(int depth); //calcule l'image par lancer de rayons. Renvoie de quoi l'afficher.
	void calc_thr(QImage* im, int t); //calcul d'un seul thread.

	// v   à utiliser si vous voulez tester un rendu de la scène avant d'implémenter un bvh.
	Vec3 ColorRay(const Vec3& P, const Vec3& V, int depth);
	Vec3 ColorRayBVH(const Vec3& P, const Vec3& V, int depth);
};

Vec3 transfo_point(Vec3 p, Mat4 trans);
Vec3 transfo_dir(Vec3 d, Mat4 trans);
void draw_prim_bvh(BVH* b);

#endif
